"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateAtomizedTasks = exports.makeAcyclic = exports.findCycle = void 0;
const output_1 = require("../utils/output");
function _findCycle(graph, id, visited, path) {
    if (visited[id])
        return null;
    visited[id] = true;
    for (const d of graph.dependencies[id]) {
        if (path.includes(d))
            return [...path, d];
        const cycle = _findCycle(graph, d, visited, [...path, d]);
        if (cycle)
            return cycle;
    }
    return null;
}
function findCycle(graph) {
    const visited = {};
    for (const t of Object.keys(graph.dependencies)) {
        visited[t] = false;
    }
    for (const t of Object.keys(graph.dependencies)) {
        const cycle = _findCycle(graph, t, visited, [t]);
        if (cycle)
            return cycle;
    }
    return null;
}
exports.findCycle = findCycle;
function _makeAcyclic(graph, id, visited, path) {
    if (visited[id])
        return;
    visited[id] = true;
    const deps = graph.dependencies[id];
    for (const d of [...deps]) {
        if (path.includes(d)) {
            deps.splice(deps.indexOf(d), 1);
        }
        else {
            _makeAcyclic(graph, d, visited, [...path, d]);
        }
    }
    return null;
}
function makeAcyclic(graph) {
    const visited = {};
    for (const t of Object.keys(graph.dependencies)) {
        visited[t] = false;
    }
    for (const t of Object.keys(graph.dependencies)) {
        _makeAcyclic(graph, t, visited, [t]);
    }
    graph.roots = Object.keys(graph.dependencies).filter((t) => graph.dependencies[t].length === 0);
}
exports.makeAcyclic = makeAcyclic;
function validateAtomizedTasks(taskGraph, projectGraph) {
    if (process.env['NX_SKIP_ATOMIZER_VALIDATION']) {
        return;
    }
    const tasksWithAtomizer = Object.values(taskGraph.tasks).filter((task) => projectGraph.nodes[task.target.project]?.data?.targets?.[task.target.target]?.metadata?.nonAtomizedTarget !== undefined);
    const isInDTE = process.env['NX_CLOUD_DISTRIBUTED_EXECUTION_ID'] ||
        process.env['NX_AGENT_NAME'];
    if (tasksWithAtomizer.length > 0 && !isInDTE) {
        const linkLine = 'Learn more at https://nx.dev/ci/features/split-e2e-tasks#use-atomizer-only-with-nx-cloud-distribution';
        if (tasksWithAtomizer.length === 1) {
            output_1.output.error({
                title: `The ${tasksWithAtomizer[0].id} task uses the atomizer and should only be run with Nx Cloud distribution.`,
                bodyLines: [linkLine],
            });
        }
        else {
            output_1.output.error({
                title: `The following tasks use the atomizer and should only be run with Nx Cloud distribution:`,
                bodyLines: [
                    `${tasksWithAtomizer.map((task) => task.id).join(', ')}`,
                    linkLine,
                ],
            });
        }
        process.exit(1);
    }
}
exports.validateAtomizedTasks = validateAtomizedTasks;
